@@ -19,7 +19,7 @@
 ## You should have received a copy of the GNU General Public License
 ## along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import pprint, sys, csv, math
+import pprint, sys, csv, math, inspect
 
 from .settings import settings
 from .util import cum_prob, frange
@@ -63,23 +63,49 @@ class Formatter(object):
             self.output = output
 
     def format(self, results):
-        sys.stderr.write("No output formatter selected.\nTest data is in %s (use with -i to format).\n" % results.dump_file)
+        sys.stderr.write("No output formatter selected.\nTest data is in %s (use with -i to format).\n" % results[0].dump_file)
 
 DefaultFormatter = Formatter
 
+class TableFormatter(Formatter):
 
-class OrgTableFormatter(Formatter):
+    def get_header(self, results):
+        name = results[0].meta("NAME")
+        keys = list(settings.DATA_SETS.keys())
+        header_row = [name]
+
+        if len(results) > 1:
+            for r in results:
+                header_row += [k + ' - ' + r.meta("TITLE") for k in keys]
+        else:
+            header_row += keys
+        return header_row
+
+    def combine_results(self, results):
+        """Generator to combine several result sets into one list of rows, by
+        concatenating them."""
+        keys = list(settings.DATA_SETS.keys())
+        for row in list(zip(*[list(r.zipped(keys)) for r in results])):
+            out_row = [row[0][0]]
+            for r in row:
+                if r[0] != out_row[0]:
+                    raise RuntimeError("x-value mismatch: %s/%s. Incompatible data sets?" % (out_row[0], r[0]))
+                out_row += r[1:]
+            yield out_row
+
+
+class OrgTableFormatter(TableFormatter):
     """Format the output for an Org mode table. The formatter is pretty crude
     and does not align the table properly, but it should be sufficient to create
     something that Org mode can correctly realign."""
 
     def format(self, results):
-        name = results.meta("NAME")
+        name = results[0].meta("NAME")
 
-        if not results:
+        if not results[0]:
             self.output.write(str(name) + " -- empty\n")
-        keys = list(settings.DATA_SETS.keys())
-        header_row = [name] + keys
+            return
+        header_row = self.get_header(results)
         self.output.write("| " + " | ".join(header_row) + " |\n")
         self.output.write("|-" + "-+-".join(["-"*len(i) for i in header_row]) + "-|\n")
 
@@ -88,25 +114,22 @@ class OrgTableFormatter(Formatter):
                 return "%.2f" % item
             return str(item)
 
-        for row in results.zipped(keys):
+        for row in self.combine_results(results):
             self.output.write("| ")
             self.output.write(" | ".join(map(format_item, row)))
             self.output.write(" |\n")
 
 
 
-class CsvFormatter(Formatter):
+class CsvFormatter(TableFormatter):
     """Format the output as csv."""
 
     def format(self, results):
-        name = results.meta("NAME")
-
-        if not results:
+        if not results[0]:
             return
 
         writer = csv.writer(self.output)
-        keys = list(settings.DATA_SETS.keys())
-        header_row = [name] + keys
+        header_row = self.get_header(results)
         writer.writerow(header_row)
 
         def format_item(item):
@@ -114,7 +137,7 @@ class CsvFormatter(Formatter):
                 return ""
             return str(item)
 
-        for row in results.zipped(keys):
+        for row in self.combine_results(results):
             writer.writerow(list(map(format_item, row)))
 
 class PlotFormatter(Formatter):
@@ -157,6 +180,7 @@ class PlotFormatter(Formatter):
 
     def _init_plots(self):
         self.config = self._load_plotconfig(settings.PLOT)
+        self.configs = [self.config]
         getattr(self, '_init_%s_plot' % self.config['type'])()
 
     def _init_timeseries_plot(self, config=None, axis=None):
@@ -208,6 +232,8 @@ class PlotFormatter(Formatter):
         axis.set_ylabel('Cumulative probability')
         axis.set_ylim(0,1)
         config['axes'] = [axis]
+        self.medians = []
+        self.min_vals = []
 
 
     def _init_meta_plot(self):
@@ -221,7 +247,7 @@ class PlotFormatter(Formatter):
                 axis.set_xlabel("")
 
 
-    def _do_timeseries_plot(self, results, config=None, axis=None):
+    def _do_timeseries_plot(self, results, config=None, axis=None, postfix=""):
         if axis is None:
             axis = self.plt.gca()
         if config is None:
@@ -242,6 +268,9 @@ class PlotFormatter(Formatter):
                 if k in s:
                     kwargs[k] = s[k]
 
+            if 'label' in kwargs:
+                kwargs['label']+=postfix
+
             y_values = results.series(s['data'], smooth)
             if 'axis' in s and s['axis'] == 2:
                 a = 1
@@ -262,9 +291,7 @@ class PlotFormatter(Formatter):
         for a in range(len(config['axes'])):
             self._do_scaling(config['axes'][a], data[a], btm, top)
 
-        self._do_legend(config)
-
-    def _do_cdf_plot(self, results, config=None, axis=None):
+    def _do_cdf_plot(self, results, config=None, axis=None, postfix=""):
         if axis is None:
             axis = self.plt.gca()
         if config is None:
@@ -281,6 +308,8 @@ class PlotFormatter(Formatter):
                 start,end = config['cutoff']
                 s_data = s_data[int(start/settings.STEP_SIZE):-int(end/settings.STEP_SIZE)]
             d = sorted([x for x in s_data if x is not None])
+            self.medians.append(self.np.median(d))
+            self.min_vals.append(min(d))
             max_value = max([max_value]+d)
             data.append(d)
 
@@ -292,6 +321,7 @@ class PlotFormatter(Formatter):
 
         x_values = list(frange(0, max_value, 0.1))
 
+
         for i,s in enumerate(config['series']):
             if not data[i]:
                 continue
@@ -299,54 +329,85 @@ class PlotFormatter(Formatter):
             for k in PLOT_KWARGS:
                 if k in s:
                     kwargs[k] = s[k]
+            if 'label' in kwargs:
+                kwargs['label']+=postfix
             axis.plot(x_values,
                       [cum_prob(data[i], point) for point in x_values],
                       **kwargs)
-        self._do_legend(config)
 
+        if max(self.medians)/min(self.medians) > 10.0:
+            # More than an order of magnitude difference; switch to log scale
+            axis.set_xscale('log')
+            min_val = min(self.min_vals)
+            if min_val > 10:
+                min_val -= min_val%10 # nearest value divisible by 10
+            axis.set_xlim(left=min_val)
 
-    def _do_meta_plot(self, results):
+    def _do_meta_plot(self, results, postfix=""):
         for i,config in enumerate(self.configs):
-            getattr(self, '_do_%s_plot' % config['type'])(results, config=config)
+            getattr(self, '_do_%s_plot' % config['type'])(results, config=config, postfix=postfix)
 
     def format(self, results):
-        if not results:
+        if not results[0]:
             return
 
-        getattr(self, '_do_%s_plot' % self.config['type'])(results)
+        if len(results) > 1:
+            for r in results:
+                getattr(self, '_do_%s_plot' % self.config['type'])(r, postfix=" - "+r.meta("TITLE"))
+            skip_title = True
+        else:
+            getattr(self, '_do_%s_plot' % self.config['type'])(results[0])
+            skip_title = False
 
-        self._annotate_plot()
+        artists = []
+        for c in self.configs:
+            artists += self._do_legend(c)
+
+        artists += self._annotate_plot(skip_title)
 
         # Since outputting image data to stdout does not make sense, we launch
         # the interactive matplotlib viewer if stdout is set for output.
         # Otherwise, the filename is passed to matplotlib, which selects an
         # appropriate output format based on the file name.
         if self.output == "-":
+            # For the interactive viewer there's no bbox_extra_artists, so we
+            # need to reduce the axis sizes to make room for the legend (which
+            # might still be slightly cut off).
+            for a in reduce(lambda x,y:x+y, [i['axes'] for i in self.configs]):
+                box = a.get_position()
+                a.set_position([box.x0, box.y0, box.width * 0.8, box.height])
             self.plt.show()
         else:
-            self.plt.savefig(self.output)
+            self.plt.savefig(self.output, bbox_extra_artists=artists, bbox_inches='tight')
 
 
-    def _annotate_plot(self):
-        plot_title = settings.DESCRIPTION
-        if 'description' in self.config:
-            plot_title += "\n" + self.config['description']
-        if settings.TITLE:
-            plot_title += "\n" + settings.TITLE
-        self.plt.suptitle(plot_title, fontsize=14)
+    def _annotate_plot(self, skip_title=False):
+        titles = []
+        if settings.PRINT_TITLE:
+            plot_title = settings.DESCRIPTION
+            if 'description' in self.config:
+                plot_title += "\n" + self.config['description']
+            if settings.TITLE and not skip_title:
+                plot_title += " - " + settings.TITLE
+            titles.append(self.plt.suptitle(plot_title, fontsize=14))
 
-        annotation_string = "Local/remote: %s/%s - Time: %s - Length/step: %ds/%.2fs" % (
-            settings.LOCAL_HOST, settings.HOST,
-            settings.TIME,
-            settings.LENGTH, settings.STEP_SIZE)
-        self.plt.suptitle(annotation_string,
-                          x=0.5,
-                          y=0.005,
-                          horizontalalignment='center',
-                          verticalalignment='bottom',
-                          fontsize=8)
+        if settings.ANNOTATE:
+            annotation_string = "Local/remote: %s/%s - Time: %s - Length/step: %ds/%.2fs" % (
+                settings.LOCAL_HOST, settings.HOST,
+                settings.TIME,
+                settings.LENGTH, settings.STEP_SIZE)
+            titles.append(self.plt.suptitle(annotation_string,
+                                            x=0.5,
+                                            y=0.005,
+                                            horizontalalignment='center',
+                                            verticalalignment='bottom',
+                                            fontsize=8))
+        return titles
+
+    def _do_legend(self, config, postfix=""):
+        if not settings.PRINT_LEGEND:
+            return []
 
-    def _do_legend(self, config):
         axes = config['axes']
 
         # Each axis has a set of handles/labels for the legend; combine them
@@ -355,22 +416,37 @@ class PlotFormatter(Formatter):
         handles, labels = reduce(lambda x,y:(x[0]+y[0], x[1]+y[1]),
                                  [a.get_legend_handles_labels() for a in axes])
 
-        # Shrink the current subplot by 20% in the horizontal direction, and
-        # place the legend on the right of the plot.
-        for a in axes:
-            box = a.get_position()
-            a.set_position([box.x0, box.y0, box.width * 0.8, box.height])
+        kwargs = {}
+        if 'legend_title' in config:
+            kwargs['title'] = config['legend_title']
 
 
-            kwargs = {}
-            if 'legend_title' in config:
-                kwargs['title'] = config['legend_title']
+        if len(axes) > 1:
+            offset_x = 1.09
+        else:
+            offset_x = 1.02
 
-            a.legend(handles, labels,
-                     bbox_to_anchor=(1.05, 1.0),
-                     loc='upper left', borderaxespad=0.,
-                     prop={'size':'small'},
-                     **kwargs)
+        legends = []
+        l = axes[0].legend(handles, labels,
+                                bbox_to_anchor=(offset_x, 1.0),
+                                loc='upper left', borderaxespad=0.,
+                                prop={'size':'small'},
+                                **kwargs)
+
+        # Work around a bug in older versions of matplotlib where the
+        # legend.get_window_extent method does not take any arguments, leading
+        # to a crash when using bbox_extra_artists when saving the figure
+        #
+        # Simply check for either the right number of args, or a vararg
+        # specification, and if they are not present, attempt to monkey-patch
+        # the method if it does not accept any arguments.
+        a,v,_,_ = inspect.getargspec(l.get_window_extent)
+        if len(a) < 2 or v is None:
+            def get_window_extent(*args, **kwargs):
+                return l.legendPatch.get_window_extent(*args, **kwargs)
+            l.get_window_extent = get_window_extent
+        legends.append(l)
+        return legends
 
     def _do_scaling(self, axis, data, btm, top):
         """Scale the axis to the selected bottom/top percentile"""
@@ -378,3 +454,5 @@ class PlotFormatter(Formatter):
         top_percentile = self.np.percentile(data, top)*1.05
         btm_percentile = self.np.percentile(data, btm)*0.95
         axis.set_ylim(ymin=btm_percentile, ymax=top_percentile)
+        if top_percentile/btm_percentile > 20.0:
+            axis.set_yscale('log')
