@@ -19,7 +19,7 @@
 ## You should have received a copy of the GNU General Public License
 ## along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
-import sys, os, optparse, socket, gzip
+import sys, os, optparse, socket
 
 from datetime import datetime
 
@@ -42,13 +42,16 @@ DEFAULT_SETTINGS = {
     'FORMAT': 'default',
     'TITLE': '',
     'LOG_FILE': None,
-    'INPUT': None,
+    'INPUT': [],
     'DESCRIPTION': 'No description',
     'PLOTS': {},
     'IP_VERSION': None,
     'DELAY': 5,
     'TIME': datetime.now(),
     'SCALE_DATA': [],
+    'ANNOTATE': True,
+    'PRINT_TITLE': True,
+    'PRINT_LEGEND': True,
     }
 
 TEST_PATH = os.path.join(DATA_DIR, 'tests')
@@ -124,15 +127,25 @@ class TestEnvironment(object):
         self.execute(os.path.join(TEST_PATH, name))
 
     def require_host_count(self, count):
-        if len(self.env['HOSTS']) < count and not self.env['INPUT']:
-            raise RuntimeError("Need %d hosts, only %d specified" % (count, len(self.env['HOSTS'])))
+        if len(self.env['HOSTS']) < count:
+            if 'DEFAULTS' in self.env and 'HOSTS' in self.env['DEFAULTS'] and self.env['DEFAULTS']['HOSTS']:
+                # If a default HOSTS list is set, populate the HOSTS list with
+                # values from this list, repeating as necessary up to count
+                def_hosts = self.env['DEFAULTS']['HOSTS']
+                host_c = len(self.env['HOSTS'])
+                missing_c = count-host_c
+                self.env['HOSTS'].extend((def_hosts * (missing_c//len(def_hosts)+1))[:missing_c])
+                if not self.env['HOST']:
+                    self.env['HOST'] = self.env['HOSTS'][0]
+            else:
+                raise RuntimeError("Need %d hosts, only %d specified" % (count, len(self.env['HOSTS'])))
 
 parser = optparse.OptionParser(description='Wrapper to run concurrent netperf-style tests',
                                usage="usage: %prog [options] -H <host> test")
 
 parser.add_option("-o", "--output", action="store", type="string", dest="OUTPUT",
                   help="file to write output to (default standard out)")
-parser.add_option("-i", "--input", action="store", type="string", dest="INPUT",
+parser.add_option("-i", "--input", action="append", type="string", dest="INPUT",
                   help="file to read input from (instead of running tests)")
 parser.add_option("-f", "--format", action="store", type="string", dest="FORMAT",
                   help="select output format (plot, csv, org_table)")
@@ -166,6 +179,12 @@ parser.add_option('--list-plots', action='store_true', dest="LIST_PLOTS",
 parser.add_option("--scale-data", action="append", type="string", dest="SCALE_DATA",
                   help="additional data files to use for scaling the plot axes "
                   "(can be supplied multiple times)")
+parser.add_option("--no-annotation", action="store_false", dest="ANNOTATE",
+                  help="do not annotate plots with hosts, time and test length")
+parser.add_option("--no-legend", action="store_false", dest="PRINT_LEGEND",
+                  help="do not print plot legend")
+parser.add_option("--no-title", action="store_false", dest="PRINT_TITLE",
+                  help="do not print plot title")
 
 class Settings(optparse.Values, object):
 
@@ -174,8 +193,13 @@ class Settings(optparse.Values, object):
         if self.HOSTS:
             self.HOST = self.HOSTS[0]
 
+        self.lookup_hosts()
+
         test_env = TestEnvironment(self.__dict__)
-        s = test_env.execute(os.path.join(TEST_PATH, test_name + ".conf"))
+        filename = os.path.join(TEST_PATH, test_name + ".conf")
+        if not os.path.exists(filename):
+            raise RuntimeError("No config file found for test '%s'" % test_name)
+        s = test_env.execute(filename)
 
         for k,v in list(s.items()):
             if k == k.upper():
@@ -189,6 +213,24 @@ class Settings(optparse.Values, object):
         if not 'TOTAL_LENGTH' in s:
             self.TOTAL_LENGTH = self.LENGTH
 
+    def lookup_hosts(self):
+        """If no explicit IP version is set, do a hostname lookup and try to"""
+        version = 4
+        for h in self.HOSTS:
+            try:
+                hostnames = socket.getaddrinfo(h, None, socket.AF_UNSPEC,
+                                               socket.SOCK_STREAM)
+                if not hostnames:
+                    raise RuntimeError("Found no hostnames on lookup of %s" % h)
+                hostname = hostnames[0]
+                if hostname[0] == socket.AF_INET6:
+                    version = 6
+            except socket.gaierror as e:
+                raise RuntimeError("Hostname lookup failed for host %s: %s" % (h,e))
+
+        if self.IP_VERSION is None:
+            self.IP_VERSION = version
+
     def __setattr__(self, k, v):
         if k in DICT_SETTINGS and isinstance(v, list):
             v = OrderedDict(v)
@@ -211,53 +253,49 @@ def load():
     if hasattr(settings, 'PLOT'):
         settings.FORMAT = 'plot'
 
-    if settings.INPUT is not None:
-        try:
-            if settings.INPUT.endswith(".gz"):
-                o = gzip.open
-            else:
-                o = open
-            fp = o(settings.INPUT, 'rt')
-            results = ResultSet.load(fp)
-            settings.load_test(results.meta("NAME"))
-            settings.update(results.meta())
-        except IOError:
-            raise RuntimeError("Unable to read input file: '%s'" % settings.INPUT)
-        finally:
-            fp.close()
+    if settings.INPUT:
+        results = []
+        test_name = None
+        for filename in settings.INPUT:
+            r = ResultSet.load_file(filename)
+            if test_name is not None and test_name != r.meta("NAME"):
+                raise RuntimeError("Result sets must be from same test (found %s/%s)" % (test_name, r.meta("NAME")))
+            test_name = r.meta("NAME")
+            results.append(r)
+
+
+        settings.update(results[0].meta())
+        settings.load_test(test_name)
     else:
         if len(args) < 1:
             parser.error("Missing test name.")
 
         test_name = args[0]
 
-        if os.path.exists(test_name):
-            test_name = os.path.splitext(os.path.basename(test_file))[0]
-
         settings.load_test(test_name)
-        results = ResultSet(NAME=settings.NAME,
+        results = [ResultSet(NAME=settings.NAME,
                             HOST=settings.HOST,
+                            HOSTS=settings.HOSTS,
                             TIME=settings.TIME,
                             LOCAL_HOST=settings.LOCAL_HOST,
                             TITLE=settings.TITLE,
                             LENGTH=settings.LENGTH,
                             TOTAL_LENGTH=settings.TOTAL_LENGTH,
-                            STEP_SIZE=settings.STEP_SIZE,)
+                            STEP_SIZE=settings.STEP_SIZE,)]
 
     if settings.SCALE_DATA:
         scale_data = []
-        try:
-            for filename in settings.SCALE_DATA:
-                with open(filename) as fp:
-                    if settings.INPUT.endswith(".gz"):
-                        fp = gzip.GzipFile(fileobj=fp)
-                    r = ResultSet.load(fp)
-                    if r.meta("NAME") != settings.NAME:
-                        raise RuntimeError("Setting name mismatch between test "
-                                           "data and scale file %s" % filename)
-                    scale_data.append(r)
-        except IOError:
-            raise RuntimeError("Unable to read input file: '%s'" % settings.INPUT)
+        for filename in settings.SCALE_DATA:
+            if filename == settings.INPUT:
+                # Do not load input file twice - makes it easier to select a set
+                # of files for plot scaling and supply each one to -i without
+                # having to change the other command line options each time.
+                continue
+            r = ResultSet.load_file(filename)
+            if r.meta("NAME") != settings.NAME:
+                raise RuntimeError("Setting name mismatch between test "
+                                   "data and scale file %s" % filename)
+            scale_data.append(r)
         settings.SCALE_DATA = scale_data
 
 
